# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._servers_operations import ServersOperations
from ._replicas_operations import ReplicasOperations
from ._firewall_rules_operations import FirewallRulesOperations
from ._virtual_network_rules_operations import VirtualNetworkRulesOperations
from ._databases_operations import DatabasesOperations
from ._configurations_operations import ConfigurationsOperations
from ._server_parameters_operations import ServerParametersOperations
from ._log_files_operations import LogFilesOperations
from ._server_administrators_operations import ServerAdministratorsOperations
from ._recoverable_servers_operations import RecoverableServersOperations
from ._server_based_performance_tier_operations import ServerBasedPerformanceTierOperations
from ._location_based_performance_tier_operations import LocationBasedPerformanceTierOperations
from ._check_name_availability_operations import CheckNameAvailabilityOperations
from ._operations import Operations
from ._server_security_alert_policies_operations import ServerSecurityAlertPoliciesOperations
from ._private_endpoint_connections_operations import PrivateEndpointConnectionsOperations
from ._private_link_resources_operations import PrivateLinkResourcesOperations
from ._server_keys_operations import ServerKeysOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ServersOperations",
    "ReplicasOperations",
    "FirewallRulesOperations",
    "VirtualNetworkRulesOperations",
    "DatabasesOperations",
    "ConfigurationsOperations",
    "ServerParametersOperations",
    "LogFilesOperations",
    "ServerAdministratorsOperations",
    "RecoverableServersOperations",
    "ServerBasedPerformanceTierOperations",
    "LocationBasedPerformanceTierOperations",
    "CheckNameAvailabilityOperations",
    "Operations",
    "ServerSecurityAlertPoliciesOperations",
    "PrivateEndpointConnectionsOperations",
    "PrivateLinkResourcesOperations",
    "ServerKeysOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
