# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._api_operations import ApiOperations
from ._api_revision_operations import ApiRevisionOperations
from ._api_release_operations import ApiReleaseOperations
from ._api_operation_operations import ApiOperationOperations
from ._api_operation_policy_operations import ApiOperationPolicyOperations
from ._tag_operations import TagOperations
from ._graph_ql_api_resolver_operations import GraphQLApiResolverOperations
from ._graph_ql_api_resolver_policy_operations import GraphQLApiResolverPolicyOperations
from ._api_product_operations import ApiProductOperations
from ._api_policy_operations import ApiPolicyOperations
from ._api_schema_operations import ApiSchemaOperations
from ._api_diagnostic_operations import ApiDiagnosticOperations
from ._api_issue_operations import ApiIssueOperations
from ._api_issue_comment_operations import ApiIssueCommentOperations
from ._api_issue_attachment_operations import ApiIssueAttachmentOperations
from ._api_tag_description_operations import ApiTagDescriptionOperations
from ._operation_operations import OperationOperations
from ._api_wiki_operations import ApiWikiOperations
from ._api_wikis_operations import ApiWikisOperations
from ._api_export_operations import ApiExportOperations
from ._api_version_set_operations import ApiVersionSetOperations
from ._authorization_server_operations import AuthorizationServerOperations
from ._authorization_provider_operations import AuthorizationProviderOperations
from ._authorization_operations import AuthorizationOperations
from ._authorization_login_links_operations import AuthorizationLoginLinksOperations
from ._authorization_access_policy_operations import AuthorizationAccessPolicyOperations
from ._backend_operations import BackendOperations
from ._cache_operations import CacheOperations
from ._certificate_operations import CertificateOperations
from ._api_management_client_operations import ApiManagementClientOperationsMixin
from ._content_type_operations import ContentTypeOperations
from ._content_item_operations import ContentItemOperations
from ._deleted_services_operations import DeletedServicesOperations
from ._api_management_operations_operations import ApiManagementOperationsOperations
from ._api_management_service_skus_operations import ApiManagementServiceSkusOperations
from ._api_management_service_operations import ApiManagementServiceOperations
from ._diagnostic_operations import DiagnosticOperations
from ._email_template_operations import EmailTemplateOperations
from ._gateway_operations import GatewayOperations
from ._gateway_hostname_configuration_operations import GatewayHostnameConfigurationOperations
from ._gateway_api_operations import GatewayApiOperations
from ._gateway_certificate_authority_operations import GatewayCertificateAuthorityOperations
from ._group_operations import GroupOperations
from ._group_user_operations import GroupUserOperations
from ._identity_provider_operations import IdentityProviderOperations
from ._issue_operations import IssueOperations
from ._logger_operations import LoggerOperations
from ._named_value_operations import NamedValueOperations
from ._network_status_operations import NetworkStatusOperations
from ._notification_operations import NotificationOperations
from ._notification_recipient_user_operations import NotificationRecipientUserOperations
from ._notification_recipient_email_operations import NotificationRecipientEmailOperations
from ._open_id_connect_provider_operations import OpenIdConnectProviderOperations
from ._outbound_network_dependencies_endpoints_operations import OutboundNetworkDependenciesEndpointsOperations
from ._policy_operations import PolicyOperations
from ._policy_description_operations import PolicyDescriptionOperations
from ._policy_fragment_operations import PolicyFragmentOperations
from ._portal_config_operations import PortalConfigOperations
from ._portal_revision_operations import PortalRevisionOperations
from ._portal_settings_operations import PortalSettingsOperations
from ._sign_in_settings_operations import SignInSettingsOperations
from ._sign_up_settings_operations import SignUpSettingsOperations
from ._delegation_settings_operations import DelegationSettingsOperations
from ._private_endpoint_connection_operations import PrivateEndpointConnectionOperations
from ._product_operations import ProductOperations
from ._product_api_operations import ProductApiOperations
from ._product_group_operations import ProductGroupOperations
from ._product_subscriptions_operations import ProductSubscriptionsOperations
from ._product_policy_operations import ProductPolicyOperations
from ._product_wiki_operations import ProductWikiOperations
from ._product_wikis_operations import ProductWikisOperations
from ._quota_by_counter_keys_operations import QuotaByCounterKeysOperations
from ._quota_by_period_keys_operations import QuotaByPeriodKeysOperations
from ._region_operations import RegionOperations
from ._reports_operations import ReportsOperations
from ._global_schema_operations import GlobalSchemaOperations
from ._tenant_settings_operations import TenantSettingsOperations
from ._api_management_skus_operations import ApiManagementSkusOperations
from ._subscription_operations import SubscriptionOperations
from ._tag_resource_operations import TagResourceOperations
from ._tenant_access_operations import TenantAccessOperations
from ._tenant_access_git_operations import TenantAccessGitOperations
from ._tenant_configuration_operations import TenantConfigurationOperations
from ._user_operations import UserOperations
from ._user_group_operations import UserGroupOperations
from ._user_subscription_operations import UserSubscriptionOperations
from ._user_identities_operations import UserIdentitiesOperations
from ._user_confirmation_password_operations import UserConfirmationPasswordOperations
from ._documentation_operations import DocumentationOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ApiOperations",
    "ApiRevisionOperations",
    "ApiReleaseOperations",
    "ApiOperationOperations",
    "ApiOperationPolicyOperations",
    "TagOperations",
    "GraphQLApiResolverOperations",
    "GraphQLApiResolverPolicyOperations",
    "ApiProductOperations",
    "ApiPolicyOperations",
    "ApiSchemaOperations",
    "ApiDiagnosticOperations",
    "ApiIssueOperations",
    "ApiIssueCommentOperations",
    "ApiIssueAttachmentOperations",
    "ApiTagDescriptionOperations",
    "OperationOperations",
    "ApiWikiOperations",
    "ApiWikisOperations",
    "ApiExportOperations",
    "ApiVersionSetOperations",
    "AuthorizationServerOperations",
    "AuthorizationProviderOperations",
    "AuthorizationOperations",
    "AuthorizationLoginLinksOperations",
    "AuthorizationAccessPolicyOperations",
    "BackendOperations",
    "CacheOperations",
    "CertificateOperations",
    "ApiManagementClientOperationsMixin",
    "ContentTypeOperations",
    "ContentItemOperations",
    "DeletedServicesOperations",
    "ApiManagementOperationsOperations",
    "ApiManagementServiceSkusOperations",
    "ApiManagementServiceOperations",
    "DiagnosticOperations",
    "EmailTemplateOperations",
    "GatewayOperations",
    "GatewayHostnameConfigurationOperations",
    "GatewayApiOperations",
    "GatewayCertificateAuthorityOperations",
    "GroupOperations",
    "GroupUserOperations",
    "IdentityProviderOperations",
    "IssueOperations",
    "LoggerOperations",
    "NamedValueOperations",
    "NetworkStatusOperations",
    "NotificationOperations",
    "NotificationRecipientUserOperations",
    "NotificationRecipientEmailOperations",
    "OpenIdConnectProviderOperations",
    "OutboundNetworkDependenciesEndpointsOperations",
    "PolicyOperations",
    "PolicyDescriptionOperations",
    "PolicyFragmentOperations",
    "PortalConfigOperations",
    "PortalRevisionOperations",
    "PortalSettingsOperations",
    "SignInSettingsOperations",
    "SignUpSettingsOperations",
    "DelegationSettingsOperations",
    "PrivateEndpointConnectionOperations",
    "ProductOperations",
    "ProductApiOperations",
    "ProductGroupOperations",
    "ProductSubscriptionsOperations",
    "ProductPolicyOperations",
    "ProductWikiOperations",
    "ProductWikisOperations",
    "QuotaByCounterKeysOperations",
    "QuotaByPeriodKeysOperations",
    "RegionOperations",
    "ReportsOperations",
    "GlobalSchemaOperations",
    "TenantSettingsOperations",
    "ApiManagementSkusOperations",
    "SubscriptionOperations",
    "TagResourceOperations",
    "TenantAccessOperations",
    "TenantAccessGitOperations",
    "TenantConfigurationOperations",
    "UserOperations",
    "UserGroupOperations",
    "UserSubscriptionOperations",
    "UserIdentitiesOperations",
    "UserConfirmationPasswordOperations",
    "DocumentationOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
