# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._backup_resource_storage_configs_non_crr_operations import BackupResourceStorageConfigsNonCRROperations  # type: ignore
from ._protection_intent_operations import ProtectionIntentOperations  # type: ignore
from ._backup_status_operations import BackupStatusOperations  # type: ignore
from ._feature_support_operations import FeatureSupportOperations  # type: ignore
from ._backup_protection_intent_operations import BackupProtectionIntentOperations  # type: ignore
from ._backup_usage_summaries_operations import BackupUsageSummariesOperations  # type: ignore
from ._operations import Operations  # type: ignore
from ._backup_resource_vault_configs_operations import BackupResourceVaultConfigsOperations  # type: ignore
from ._backup_resource_encryption_configs_operations import BackupResourceEncryptionConfigsOperations  # type: ignore
from ._private_endpoint_connection_operations import PrivateEndpointConnectionOperations  # type: ignore
from ._private_endpoint_operations import PrivateEndpointOperations  # type: ignore
from ._recovery_services_backup_client_operations import RecoveryServicesBackupClientOperationsMixin  # type: ignore
from ._bms_prepare_data_move_operation_result_operations import BMSPrepareDataMoveOperationResultOperations  # type: ignore
from ._protected_items_operations import ProtectedItemsOperations  # type: ignore
from ._protected_item_operation_results_operations import ProtectedItemOperationResultsOperations  # type: ignore
from ._recovery_points_operations import RecoveryPointsOperations  # type: ignore
from ._restores_operations import RestoresOperations  # type: ignore
from ._backup_policies_operations import BackupPoliciesOperations  # type: ignore
from ._protection_policies_operations import ProtectionPoliciesOperations  # type: ignore
from ._protection_policy_operation_results_operations import ProtectionPolicyOperationResultsOperations  # type: ignore
from ._backup_jobs_operations import BackupJobsOperations  # type: ignore
from ._job_details_operations import JobDetailsOperations  # type: ignore
from ._job_cancellations_operations import JobCancellationsOperations  # type: ignore
from ._job_operation_results_operations import JobOperationResultsOperations  # type: ignore
from ._export_jobs_operation_results_operations import ExportJobsOperationResultsOperations  # type: ignore
from ._jobs_operations import JobsOperations  # type: ignore
from ._backup_protected_items_operations import BackupProtectedItemsOperations  # type: ignore
from ._operation_operations import OperationOperations  # type: ignore
from ._validate_operation_operations import ValidateOperationOperations  # type: ignore
from ._validate_operation_results_operations import ValidateOperationResultsOperations  # type: ignore
from ._validate_operation_statuses_operations import ValidateOperationStatusesOperations  # type: ignore
from ._backup_engines_operations import BackupEnginesOperations  # type: ignore
from ._protection_container_refresh_operation_results_operations import ProtectionContainerRefreshOperationResultsOperations  # type: ignore
from ._protectable_containers_operations import ProtectableContainersOperations  # type: ignore
from ._protection_containers_operations import ProtectionContainersOperations  # type: ignore
from ._backup_workload_items_operations import BackupWorkloadItemsOperations  # type: ignore
from ._protection_container_operation_results_operations import ProtectionContainerOperationResultsOperations  # type: ignore
from ._backups_operations import BackupsOperations  # type: ignore
from ._protected_item_operation_statuses_operations import ProtectedItemOperationStatusesOperations  # type: ignore
from ._item_level_recovery_connections_operations import ItemLevelRecoveryConnectionsOperations  # type: ignore
from ._backup_operation_results_operations import BackupOperationResultsOperations  # type: ignore
from ._backup_operation_statuses_operations import BackupOperationStatusesOperations  # type: ignore
from ._protection_policy_operation_statuses_operations import ProtectionPolicyOperationStatusesOperations  # type: ignore
from ._backup_protectable_items_operations import BackupProtectableItemsOperations  # type: ignore
from ._backup_protection_containers_operations import BackupProtectionContainersOperations  # type: ignore
from ._deleted_protection_containers_operations import DeletedProtectionContainersOperations  # type: ignore
from ._security_pins_operations import SecurityPINsOperations  # type: ignore
from ._recovery_points_recommended_for_move_operations import RecoveryPointsRecommendedForMoveOperations  # type: ignore
from ._resource_guard_proxies_operations import ResourceGuardProxiesOperations  # type: ignore
from ._resource_guard_proxy_operations import ResourceGuardProxyOperations  # type: ignore
from ._fetch_tiering_cost_operations import FetchTieringCostOperations  # type: ignore
from ._get_tiering_cost_operation_result_operations import GetTieringCostOperationResultOperations  # type: ignore
from ._tiering_cost_operation_status_operations import TieringCostOperationStatusOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "BackupResourceStorageConfigsNonCRROperations",
    "ProtectionIntentOperations",
    "BackupStatusOperations",
    "FeatureSupportOperations",
    "BackupProtectionIntentOperations",
    "BackupUsageSummariesOperations",
    "Operations",
    "BackupResourceVaultConfigsOperations",
    "BackupResourceEncryptionConfigsOperations",
    "PrivateEndpointConnectionOperations",
    "PrivateEndpointOperations",
    "RecoveryServicesBackupClientOperationsMixin",
    "BMSPrepareDataMoveOperationResultOperations",
    "ProtectedItemsOperations",
    "ProtectedItemOperationResultsOperations",
    "RecoveryPointsOperations",
    "RestoresOperations",
    "BackupPoliciesOperations",
    "ProtectionPoliciesOperations",
    "ProtectionPolicyOperationResultsOperations",
    "BackupJobsOperations",
    "JobDetailsOperations",
    "JobCancellationsOperations",
    "JobOperationResultsOperations",
    "ExportJobsOperationResultsOperations",
    "JobsOperations",
    "BackupProtectedItemsOperations",
    "OperationOperations",
    "ValidateOperationOperations",
    "ValidateOperationResultsOperations",
    "ValidateOperationStatusesOperations",
    "BackupEnginesOperations",
    "ProtectionContainerRefreshOperationResultsOperations",
    "ProtectableContainersOperations",
    "ProtectionContainersOperations",
    "BackupWorkloadItemsOperations",
    "ProtectionContainerOperationResultsOperations",
    "BackupsOperations",
    "ProtectedItemOperationStatusesOperations",
    "ItemLevelRecoveryConnectionsOperations",
    "BackupOperationResultsOperations",
    "BackupOperationStatusesOperations",
    "ProtectionPolicyOperationStatusesOperations",
    "BackupProtectableItemsOperations",
    "BackupProtectionContainersOperations",
    "DeletedProtectionContainersOperations",
    "SecurityPINsOperations",
    "RecoveryPointsRecommendedForMoveOperations",
    "ResourceGuardProxiesOperations",
    "ResourceGuardProxyOperations",
    "FetchTieringCostOperations",
    "GetTieringCostOperationResultOperations",
    "TieringCostOperationStatusOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
