# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import AppInsightsReference
    from ._models_py3 import AutoScaleSettings
    from ._models_py3 import AzureBlobFileSystemReference
    from ._models_py3 import AzureFileShareReference
    from ._models_py3 import AzureStorageCredentialsInfo
    from ._models_py3 import BatchAIError
    from ._models_py3 import CNTKsettings
    from ._models_py3 import Caffe2Settings
    from ._models_py3 import CaffeSettings
    from ._models_py3 import ChainerSettings
    from ._models_py3 import CloudErrorBody
    from ._models_py3 import Cluster
    from ._models_py3 import ClusterCreateParameters
    from ._models_py3 import ClusterListResult
    from ._models_py3 import ClusterUpdateParameters
    from ._models_py3 import ClustersListByWorkspaceOptions
    from ._models_py3 import ContainerSettings
    from ._models_py3 import CustomMpiSettings
    from ._models_py3 import CustomToolkitSettings
    from ._models_py3 import DataDisks
    from ._models_py3 import EnvironmentVariable
    from ._models_py3 import EnvironmentVariableWithSecretValue
    from ._models_py3 import Experiment
    from ._models_py3 import ExperimentListResult
    from ._models_py3 import ExperimentsListByWorkspaceOptions
    from ._models_py3 import File
    from ._models_py3 import FileListResult
    from ._models_py3 import FileServer
    from ._models_py3 import FileServerCreateParameters
    from ._models_py3 import FileServerListResult
    from ._models_py3 import FileServerReference
    from ._models_py3 import FileServersListByWorkspaceOptions
    from ._models_py3 import HorovodSettings
    from ._models_py3 import ImageReference
    from ._models_py3 import ImageSourceRegistry
    from ._models_py3 import InputDirectory
    from ._models_py3 import Job
    from ._models_py3 import JobBasePropertiesConstraints
    from ._models_py3 import JobCreateParameters
    from ._models_py3 import JobListResult
    from ._models_py3 import JobPreparation
    from ._models_py3 import JobPropertiesConstraints
    from ._models_py3 import JobPropertiesExecutionInfo
    from ._models_py3 import JobsListByExperimentOptions
    from ._models_py3 import JobsListOutputFilesOptions
    from ._models_py3 import KeyVaultSecretReference
    from ._models_py3 import ListUsagesResult
    from ._models_py3 import ManualScaleSettings
    from ._models_py3 import MountSettings
    from ._models_py3 import MountVolumes
    from ._models_py3 import NameValuePair
    from ._models_py3 import NodeSetup
    from ._models_py3 import NodeStateCounts
    from ._models_py3 import Operation
    from ._models_py3 import OperationDisplay
    from ._models_py3 import OperationListResult
    from ._models_py3 import OutputDirectory
    from ._models_py3 import PerformanceCountersSettings
    from ._models_py3 import PrivateRegistryCredentials
    from ._models_py3 import ProxyResource
    from ._models_py3 import PyTorchSettings
    from ._models_py3 import RemoteLoginInformation
    from ._models_py3 import RemoteLoginInformationListResult
    from ._models_py3 import Resource
    from ._models_py3 import ResourceId
    from ._models_py3 import ScaleSettings
    from ._models_py3 import SetupTask
    from ._models_py3 import SshConfiguration
    from ._models_py3 import TensorFlowSettings
    from ._models_py3 import UnmanagedFileSystemReference
    from ._models_py3 import Usage
    from ._models_py3 import UsageName
    from ._models_py3 import UserAccountSettings
    from ._models_py3 import VirtualMachineConfiguration
    from ._models_py3 import Workspace
    from ._models_py3 import WorkspaceCreateParameters
    from ._models_py3 import WorkspaceListResult
    from ._models_py3 import WorkspaceUpdateParameters
    from ._models_py3 import WorkspacesListByResourceGroupOptions
    from ._models_py3 import WorkspacesListOptions
except (SyntaxError, ImportError):
    from ._models import AppInsightsReference  # type: ignore
    from ._models import AutoScaleSettings  # type: ignore
    from ._models import AzureBlobFileSystemReference  # type: ignore
    from ._models import AzureFileShareReference  # type: ignore
    from ._models import AzureStorageCredentialsInfo  # type: ignore
    from ._models import BatchAIError  # type: ignore
    from ._models import CNTKsettings  # type: ignore
    from ._models import Caffe2Settings  # type: ignore
    from ._models import CaffeSettings  # type: ignore
    from ._models import ChainerSettings  # type: ignore
    from ._models import CloudErrorBody  # type: ignore
    from ._models import Cluster  # type: ignore
    from ._models import ClusterCreateParameters  # type: ignore
    from ._models import ClusterListResult  # type: ignore
    from ._models import ClusterUpdateParameters  # type: ignore
    from ._models import ClustersListByWorkspaceOptions  # type: ignore
    from ._models import ContainerSettings  # type: ignore
    from ._models import CustomMpiSettings  # type: ignore
    from ._models import CustomToolkitSettings  # type: ignore
    from ._models import DataDisks  # type: ignore
    from ._models import EnvironmentVariable  # type: ignore
    from ._models import EnvironmentVariableWithSecretValue  # type: ignore
    from ._models import Experiment  # type: ignore
    from ._models import ExperimentListResult  # type: ignore
    from ._models import ExperimentsListByWorkspaceOptions  # type: ignore
    from ._models import File  # type: ignore
    from ._models import FileListResult  # type: ignore
    from ._models import FileServer  # type: ignore
    from ._models import FileServerCreateParameters  # type: ignore
    from ._models import FileServerListResult  # type: ignore
    from ._models import FileServerReference  # type: ignore
    from ._models import FileServersListByWorkspaceOptions  # type: ignore
    from ._models import HorovodSettings  # type: ignore
    from ._models import ImageReference  # type: ignore
    from ._models import ImageSourceRegistry  # type: ignore
    from ._models import InputDirectory  # type: ignore
    from ._models import Job  # type: ignore
    from ._models import JobBasePropertiesConstraints  # type: ignore
    from ._models import JobCreateParameters  # type: ignore
    from ._models import JobListResult  # type: ignore
    from ._models import JobPreparation  # type: ignore
    from ._models import JobPropertiesConstraints  # type: ignore
    from ._models import JobPropertiesExecutionInfo  # type: ignore
    from ._models import JobsListByExperimentOptions  # type: ignore
    from ._models import JobsListOutputFilesOptions  # type: ignore
    from ._models import KeyVaultSecretReference  # type: ignore
    from ._models import ListUsagesResult  # type: ignore
    from ._models import ManualScaleSettings  # type: ignore
    from ._models import MountSettings  # type: ignore
    from ._models import MountVolumes  # type: ignore
    from ._models import NameValuePair  # type: ignore
    from ._models import NodeSetup  # type: ignore
    from ._models import NodeStateCounts  # type: ignore
    from ._models import Operation  # type: ignore
    from ._models import OperationDisplay  # type: ignore
    from ._models import OperationListResult  # type: ignore
    from ._models import OutputDirectory  # type: ignore
    from ._models import PerformanceCountersSettings  # type: ignore
    from ._models import PrivateRegistryCredentials  # type: ignore
    from ._models import ProxyResource  # type: ignore
    from ._models import PyTorchSettings  # type: ignore
    from ._models import RemoteLoginInformation  # type: ignore
    from ._models import RemoteLoginInformationListResult  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import ResourceId  # type: ignore
    from ._models import ScaleSettings  # type: ignore
    from ._models import SetupTask  # type: ignore
    from ._models import SshConfiguration  # type: ignore
    from ._models import TensorFlowSettings  # type: ignore
    from ._models import UnmanagedFileSystemReference  # type: ignore
    from ._models import Usage  # type: ignore
    from ._models import UsageName  # type: ignore
    from ._models import UserAccountSettings  # type: ignore
    from ._models import VirtualMachineConfiguration  # type: ignore
    from ._models import Workspace  # type: ignore
    from ._models import WorkspaceCreateParameters  # type: ignore
    from ._models import WorkspaceListResult  # type: ignore
    from ._models import WorkspaceUpdateParameters  # type: ignore
    from ._models import WorkspacesListByResourceGroupOptions  # type: ignore
    from ._models import WorkspacesListOptions  # type: ignore

from ._batch_ai_enums import (
    AllocationState,
    CachingType,
    DeallocationOption,
    ExecutionState,
    FileServerProvisioningState,
    FileType,
    JobPriority,
    ProvisioningState,
    StorageAccountType,
    ToolType,
    UsageUnit,
    VmPriority,
)

__all__ = [
    'AppInsightsReference',
    'AutoScaleSettings',
    'AzureBlobFileSystemReference',
    'AzureFileShareReference',
    'AzureStorageCredentialsInfo',
    'BatchAIError',
    'CNTKsettings',
    'Caffe2Settings',
    'CaffeSettings',
    'ChainerSettings',
    'CloudErrorBody',
    'Cluster',
    'ClusterCreateParameters',
    'ClusterListResult',
    'ClusterUpdateParameters',
    'ClustersListByWorkspaceOptions',
    'ContainerSettings',
    'CustomMpiSettings',
    'CustomToolkitSettings',
    'DataDisks',
    'EnvironmentVariable',
    'EnvironmentVariableWithSecretValue',
    'Experiment',
    'ExperimentListResult',
    'ExperimentsListByWorkspaceOptions',
    'File',
    'FileListResult',
    'FileServer',
    'FileServerCreateParameters',
    'FileServerListResult',
    'FileServerReference',
    'FileServersListByWorkspaceOptions',
    'HorovodSettings',
    'ImageReference',
    'ImageSourceRegistry',
    'InputDirectory',
    'Job',
    'JobBasePropertiesConstraints',
    'JobCreateParameters',
    'JobListResult',
    'JobPreparation',
    'JobPropertiesConstraints',
    'JobPropertiesExecutionInfo',
    'JobsListByExperimentOptions',
    'JobsListOutputFilesOptions',
    'KeyVaultSecretReference',
    'ListUsagesResult',
    'ManualScaleSettings',
    'MountSettings',
    'MountVolumes',
    'NameValuePair',
    'NodeSetup',
    'NodeStateCounts',
    'Operation',
    'OperationDisplay',
    'OperationListResult',
    'OutputDirectory',
    'PerformanceCountersSettings',
    'PrivateRegistryCredentials',
    'ProxyResource',
    'PyTorchSettings',
    'RemoteLoginInformation',
    'RemoteLoginInformationListResult',
    'Resource',
    'ResourceId',
    'ScaleSettings',
    'SetupTask',
    'SshConfiguration',
    'TensorFlowSettings',
    'UnmanagedFileSystemReference',
    'Usage',
    'UsageName',
    'UserAccountSettings',
    'VirtualMachineConfiguration',
    'Workspace',
    'WorkspaceCreateParameters',
    'WorkspaceListResult',
    'WorkspaceUpdateParameters',
    'WorkspacesListByResourceGroupOptions',
    'WorkspacesListOptions',
    'AllocationState',
    'CachingType',
    'DeallocationOption',
    'ExecutionState',
    'FileServerProvisioningState',
    'FileType',
    'JobPriority',
    'ProvisioningState',
    'StorageAccountType',
    'ToolType',
    'UsageUnit',
    'VmPriority',
]
