# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import Attributes
from ._models_py3 import BackupSecretResult
from ._models_py3 import DeletedSecretBundle
from ._models_py3 import DeletedSecretItem
from ._models_py3 import DeletedSecretListResult
from ._models_py3 import Error
from ._models_py3 import KeyVaultError
from ._models_py3 import SecretAttributes
from ._models_py3 import SecretBundle
from ._models_py3 import SecretItem
from ._models_py3 import SecretListResult
from ._models_py3 import SecretProperties
from ._models_py3 import SecretRestoreParameters
from ._models_py3 import SecretSetParameters
from ._models_py3 import SecretUpdateParameters

from ._key_vault_client_enums import DeletionRecoveryLevel
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Attributes",
    "BackupSecretResult",
    "DeletedSecretBundle",
    "DeletedSecretItem",
    "DeletedSecretListResult",
    "Error",
    "KeyVaultError",
    "SecretAttributes",
    "SecretBundle",
    "SecretItem",
    "SecretListResult",
    "SecretProperties",
    "SecretRestoreParameters",
    "SecretSetParameters",
    "SecretUpdateParameters",
    "DeletionRecoveryLevel",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
