# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import AgreementTerms
    from ._models_py3 import ErrorResponse
    from ._models_py3 import ErrorResponseError
    from ._models_py3 import Operation
    from ._models_py3 import OperationDisplay
    from ._models_py3 import OperationListResult
    from ._models_py3 import Resource
    from ._models_py3 import SystemData
except (SyntaxError, ImportError):
    from ._models import AgreementTerms  # type: ignore
    from ._models import ErrorResponse  # type: ignore
    from ._models import ErrorResponseError  # type: ignore
    from ._models import Operation  # type: ignore
    from ._models import OperationDisplay  # type: ignore
    from ._models import OperationListResult  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import SystemData  # type: ignore

from ._marketplace_ordering_agreements_enums import (
    CreatedByType,
    OfferType,
)

__all__ = [
    'AgreementTerms',
    'ErrorResponse',
    'ErrorResponseError',
    'Operation',
    'OperationDisplay',
    'OperationListResult',
    'Resource',
    'SystemData',
    'CreatedByType',
    'OfferType',
]
