# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    AdminCredentials,
    AdminCredentialsForPatch,
    AdministratorMicrosoftEntra,
    AdministratorMicrosoftEntraAdd,
    AdministratorMicrosoftEntraProperties,
    AdministratorMicrosoftEntraPropertiesForAdd,
    AdvancedThreatProtectionSettingsModel,
    AdvancedThreatProtectionSettingsProperties,
    AuthConfig,
    AuthConfigForPatch,
    Backup,
    BackupAutomaticAndOnDemand,
    BackupAutomaticAndOnDemandProperties,
    BackupForPatch,
    BackupRequestBase,
    BackupSettings,
    BackupStoreDetails,
    BackupsLongTermRetentionOperation,
    BackupsLongTermRetentionRequest,
    BackupsLongTermRetentionResponse,
    BackupsLongTermRetentionResponseProperties,
    Capability,
    CapabilityBase,
    CapturedLog,
    CapturedLogProperties,
    CheckNameAvailabilityRequest,
    CheckNameAvailabilityResponse,
    Cluster,
    Configuration,
    ConfigurationForUpdate,
    ConfigurationProperties,
    DataEncryption,
    Database,
    DatabaseMigrationState,
    DatabaseProperties,
    DbLevelValidationStatus,
    DbServerMetadata,
    DelegatedSubnetUsage,
    ErrorAdditionalInfo,
    ErrorDetail,
    ErrorResponse,
    FastProvisioningEditionCapability,
    FirewallRule,
    FirewallRuleProperties,
    HighAvailability,
    HighAvailabilityForPatch,
    ImpactRecord,
    LogSpecification,
    LtrBackupOperationResponseProperties,
    LtrPreBackupRequest,
    LtrPreBackupResponse,
    MaintenanceWindow,
    MaintenanceWindowForPatch,
    MetricSpecification,
    MigrateNetworkStatus,
    Migration,
    MigrationNameAvailability,
    MigrationProperties,
    MigrationPropertiesForPatch,
    MigrationResourceForPatch,
    MigrationSecretParameters,
    MigrationSecretParametersForPatch,
    MigrationStatus,
    MigrationSubstateDetails,
    NameAvailabilityModel,
    NameProperty,
    Network,
    ObjectRecommendation,
    ObjectRecommendationDetails,
    ObjectRecommendationProperties,
    ObjectRecommendationPropertiesAnalyzedWorkload,
    ObjectRecommendationPropertiesImplementationDetails,
    Operation,
    OperationDisplay,
    OperationProperties,
    PrivateEndpoint,
    PrivateEndpointConnection,
    PrivateEndpointConnectionProperties,
    PrivateLinkResource,
    PrivateLinkResourceProperties,
    PrivateLinkServiceConnectionState,
    ProxyResource,
    QuotaUsage,
    Replica,
    Resource,
    RestartParameter,
    Server,
    ServerEditionCapability,
    ServerForPatch,
    ServerProperties,
    ServerPropertiesForPatch,
    ServerSku,
    ServerSkuCapability,
    ServerVersionCapability,
    ServiceSpecification,
    Sku,
    SkuForPatch,
    Storage,
    StorageEditionCapability,
    StorageMbCapability,
    StorageTierCapability,
    SupportedFeature,
    SystemData,
    TrackedResource,
    TuningOptions,
    TuningOptionsProperties,
    UserAssignedIdentity,
    UserIdentity,
    ValidationDetails,
    ValidationMessage,
    ValidationSummaryItem,
    VirtualEndpoint,
    VirtualEndpointResourceForPatch,
    VirtualEndpointResourceProperties,
    VirtualNetworkSubnetUsageModel,
    VirtualNetworkSubnetUsageParameter,
)

from ._enums import (  # type: ignore
    AzureManagedDiskPerformanceTier,
    BackupType,
    Cancel,
    CapabilityStatus,
    CheckNameAvailabilityReason,
    ConfigurationDataType,
    CreateMode,
    CreateModeForPatch,
    CreatedByType,
    DataEncryptionType,
    EncryptionKeyStatus,
    ExecutionStatus,
    FailoverMode,
    FastProvisioningSupport,
    FeatureStatus,
    GeographicallyRedundantBackup,
    GeographicallyRedundantBackupSupport,
    HighAvailabilityMode,
    HighAvailabilityState,
    IdentityType,
    LocationRestricted,
    LogicalReplicationOnSourceServer,
    MicrosoftEntraAuth,
    MigrateRolesAndPermissions,
    MigrationDatabaseState,
    MigrationListFilter,
    MigrationMode,
    MigrationNameAvailabilityReason,
    MigrationOption,
    MigrationState,
    MigrationSubstate,
    NetworkMigrationState,
    OnlineStorageResizeSupport,
    OperationOrigin,
    OverwriteDatabasesOnTargetServer,
    PasswordBasedAuth,
    PostgreSqlFlexibleServerHighAvailabilityMode,
    PostgresMajorVersion,
    PrincipalType,
    PrivateEndpointConnectionProvisioningState,
    PrivateEndpointServiceConnectionStatus,
    ReadReplicaPromoteMode,
    ReadReplicaPromoteOption,
    RecommendationTypeEnum,
    RecommendationTypeParameterEnum,
    ReplicationRole,
    ReplicationState,
    ServerPublicNetworkAccessState,
    ServerState,
    SkuTier,
    SourceType,
    SslMode,
    StartDataMigration,
    StorageAutoGrow,
    StorageAutoGrowthSupport,
    StorageType,
    ThreatProtectionName,
    ThreatProtectionState,
    TriggerCutover,
    TuningOptionParameterEnum,
    ValidationState,
    VirtualEndpointType,
    ZoneRedundantHighAvailabilityAndGeographicallyRedundantBackupSupport,
    ZoneRedundantHighAvailabilitySupport,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AdminCredentials",
    "AdminCredentialsForPatch",
    "AdministratorMicrosoftEntra",
    "AdministratorMicrosoftEntraAdd",
    "AdministratorMicrosoftEntraProperties",
    "AdministratorMicrosoftEntraPropertiesForAdd",
    "AdvancedThreatProtectionSettingsModel",
    "AdvancedThreatProtectionSettingsProperties",
    "AuthConfig",
    "AuthConfigForPatch",
    "Backup",
    "BackupAutomaticAndOnDemand",
    "BackupAutomaticAndOnDemandProperties",
    "BackupForPatch",
    "BackupRequestBase",
    "BackupSettings",
    "BackupStoreDetails",
    "BackupsLongTermRetentionOperation",
    "BackupsLongTermRetentionRequest",
    "BackupsLongTermRetentionResponse",
    "BackupsLongTermRetentionResponseProperties",
    "Capability",
    "CapabilityBase",
    "CapturedLog",
    "CapturedLogProperties",
    "CheckNameAvailabilityRequest",
    "CheckNameAvailabilityResponse",
    "Cluster",
    "Configuration",
    "ConfigurationForUpdate",
    "ConfigurationProperties",
    "DataEncryption",
    "Database",
    "DatabaseMigrationState",
    "DatabaseProperties",
    "DbLevelValidationStatus",
    "DbServerMetadata",
    "DelegatedSubnetUsage",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "FastProvisioningEditionCapability",
    "FirewallRule",
    "FirewallRuleProperties",
    "HighAvailability",
    "HighAvailabilityForPatch",
    "ImpactRecord",
    "LogSpecification",
    "LtrBackupOperationResponseProperties",
    "LtrPreBackupRequest",
    "LtrPreBackupResponse",
    "MaintenanceWindow",
    "MaintenanceWindowForPatch",
    "MetricSpecification",
    "MigrateNetworkStatus",
    "Migration",
    "MigrationNameAvailability",
    "MigrationProperties",
    "MigrationPropertiesForPatch",
    "MigrationResourceForPatch",
    "MigrationSecretParameters",
    "MigrationSecretParametersForPatch",
    "MigrationStatus",
    "MigrationSubstateDetails",
    "NameAvailabilityModel",
    "NameProperty",
    "Network",
    "ObjectRecommendation",
    "ObjectRecommendationDetails",
    "ObjectRecommendationProperties",
    "ObjectRecommendationPropertiesAnalyzedWorkload",
    "ObjectRecommendationPropertiesImplementationDetails",
    "Operation",
    "OperationDisplay",
    "OperationProperties",
    "PrivateEndpoint",
    "PrivateEndpointConnection",
    "PrivateEndpointConnectionProperties",
    "PrivateLinkResource",
    "PrivateLinkResourceProperties",
    "PrivateLinkServiceConnectionState",
    "ProxyResource",
    "QuotaUsage",
    "Replica",
    "Resource",
    "RestartParameter",
    "Server",
    "ServerEditionCapability",
    "ServerForPatch",
    "ServerProperties",
    "ServerPropertiesForPatch",
    "ServerSku",
    "ServerSkuCapability",
    "ServerVersionCapability",
    "ServiceSpecification",
    "Sku",
    "SkuForPatch",
    "Storage",
    "StorageEditionCapability",
    "StorageMbCapability",
    "StorageTierCapability",
    "SupportedFeature",
    "SystemData",
    "TrackedResource",
    "TuningOptions",
    "TuningOptionsProperties",
    "UserAssignedIdentity",
    "UserIdentity",
    "ValidationDetails",
    "ValidationMessage",
    "ValidationSummaryItem",
    "VirtualEndpoint",
    "VirtualEndpointResourceForPatch",
    "VirtualEndpointResourceProperties",
    "VirtualNetworkSubnetUsageModel",
    "VirtualNetworkSubnetUsageParameter",
    "AzureManagedDiskPerformanceTier",
    "BackupType",
    "Cancel",
    "CapabilityStatus",
    "CheckNameAvailabilityReason",
    "ConfigurationDataType",
    "CreateMode",
    "CreateModeForPatch",
    "CreatedByType",
    "DataEncryptionType",
    "EncryptionKeyStatus",
    "ExecutionStatus",
    "FailoverMode",
    "FastProvisioningSupport",
    "FeatureStatus",
    "GeographicallyRedundantBackup",
    "GeographicallyRedundantBackupSupport",
    "HighAvailabilityMode",
    "HighAvailabilityState",
    "IdentityType",
    "LocationRestricted",
    "LogicalReplicationOnSourceServer",
    "MicrosoftEntraAuth",
    "MigrateRolesAndPermissions",
    "MigrationDatabaseState",
    "MigrationListFilter",
    "MigrationMode",
    "MigrationNameAvailabilityReason",
    "MigrationOption",
    "MigrationState",
    "MigrationSubstate",
    "NetworkMigrationState",
    "OnlineStorageResizeSupport",
    "OperationOrigin",
    "OverwriteDatabasesOnTargetServer",
    "PasswordBasedAuth",
    "PostgreSqlFlexibleServerHighAvailabilityMode",
    "PostgresMajorVersion",
    "PrincipalType",
    "PrivateEndpointConnectionProvisioningState",
    "PrivateEndpointServiceConnectionStatus",
    "ReadReplicaPromoteMode",
    "ReadReplicaPromoteOption",
    "RecommendationTypeEnum",
    "RecommendationTypeParameterEnum",
    "ReplicationRole",
    "ReplicationState",
    "ServerPublicNetworkAccessState",
    "ServerState",
    "SkuTier",
    "SourceType",
    "SslMode",
    "StartDataMigration",
    "StorageAutoGrow",
    "StorageAutoGrowthSupport",
    "StorageType",
    "ThreatProtectionName",
    "ThreatProtectionState",
    "TriggerCutover",
    "TuningOptionParameterEnum",
    "ValidationState",
    "VirtualEndpointType",
    "ZoneRedundantHighAvailabilityAndGeographicallyRedundantBackupSupport",
    "ZoneRedundantHighAvailabilitySupport",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
