# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    ActivationProperties,
    Actor,
    Archive,
    ArchivePackageSourceProperties,
    ArchiveProperties,
    ArchiveUpdateParameters,
    ArchiveUpdateProperties,
    ArchiveVersion,
    ArchiveVersionProperties,
    AuthCredential,
    AzureADAuthenticationAsArmPolicy,
    CacheRule,
    CacheRuleProperties,
    CacheRuleUpdateParameters,
    CacheRuleUpdateProperties,
    CallbackConfig,
    ConnectedRegistry,
    ConnectedRegistryProperties,
    ConnectedRegistryUpdateParameters,
    ConnectedRegistryUpdateProperties,
    CredentialHealth,
    CredentialSet,
    CredentialSetProperties,
    CredentialSetUpdateParameters,
    CredentialSetUpdateProperties,
    EncryptionProperty,
    ErrorAdditionalInfo,
    ErrorDetail,
    ErrorResponse,
    Event,
    EventContent,
    EventInfo,
    EventRequestMessage,
    EventResponseMessage,
    ExportPipeline,
    ExportPipelineProperties,
    ExportPipelineTargetProperties,
    ExportPolicy,
    GarbageCollectionProperties,
    GenerateCredentialsParameters,
    GenerateCredentialsResult,
    IPRule,
    IdentityProperties,
    ImportImageParameters,
    ImportPipeline,
    ImportPipelineProperties,
    ImportPipelineSourceProperties,
    ImportSource,
    ImportSourceCredentials,
    KeyVaultProperties,
    LoggingProperties,
    LoginServerProperties,
    NetworkRuleSet,
    OperationDefinition,
    OperationDisplayDefinition,
    OperationLogSpecificationDefinition,
    OperationMetricSpecificationDefinition,
    OperationPropertiesDefinition,
    OperationServiceSpecificationDefinition,
    ParentProperties,
    PipelineRun,
    PipelineRunProperties,
    PipelineRunRequest,
    PipelineRunResponse,
    PipelineRunSourceProperties,
    PipelineRunTargetProperties,
    PipelineSourceTriggerDescriptor,
    PipelineSourceTriggerProperties,
    PipelineTriggerDescriptor,
    PipelineTriggerProperties,
    Policies,
    PrivateEndpoint,
    PrivateEndpointConnection,
    PrivateEndpointConnectionProperties,
    PrivateLinkResource,
    PrivateLinkResourceProperties,
    PrivateLinkServiceConnectionState,
    ProgressProperties,
    ProxyResource,
    QuarantinePolicy,
    RegenerateCredentialParameters,
    Registry,
    RegistryListCredentialsResult,
    RegistryNameCheckRequest,
    RegistryNameStatus,
    RegistryPassword,
    RegistryProperties,
    RegistryPropertiesUpdateParameters,
    RegistrySyncResult,
    RegistryUpdateParameters,
    RegistryUsage,
    RegistryUsageListResult,
    Replication,
    ReplicationProperties,
    ReplicationUpdateParameters,
    ReplicationUpdateParametersProperties,
    Request,
    Resource,
    RetentionPolicy,
    ScopeMap,
    ScopeMapProperties,
    ScopeMapPropertiesUpdateParameters,
    ScopeMapUpdateParameters,
    Sku,
    SoftDeletePolicy,
    Source,
    Status,
    StatusDetailProperties,
    SyncProperties,
    SyncUpdateProperties,
    SystemData,
    Target,
    TlsCertificateProperties,
    TlsProperties,
    Token,
    TokenCertificate,
    TokenCredentialsProperties,
    TokenPassword,
    TokenProperties,
    TokenUpdateParameters,
    TokenUpdateProperties,
    TrackedResource,
    TrustPolicy,
    UserIdentityProperties,
    Webhook,
    WebhookCreateParameters,
    WebhookProperties,
    WebhookPropertiesCreateParameters,
    WebhookPropertiesUpdateParameters,
    WebhookUpdateParameters,
)

from ._enums import (  # type: ignore
    Action,
    ActionsRequired,
    ActivationStatus,
    AuditLogStatus,
    AutoGeneratedDomainNameLabelScope,
    AzureADAuthenticationAsArmPolicyStatus,
    CertificateType,
    ConnectedRegistryMode,
    ConnectionState,
    ConnectionStatus,
    ContainerRegistryResourceType,
    CreatedByType,
    CredentialHealthStatus,
    CredentialName,
    DefaultAction,
    EncryptionStatus,
    EndpointProtocol,
    ExportPolicyStatus,
    ImportMode,
    LogLevel,
    MetadataSearch,
    NetworkRuleBypassOptions,
    PackageSourceType,
    PasswordName,
    PipelineOptions,
    PipelineRunSourceType,
    PipelineRunTargetType,
    PipelineSourceType,
    PolicyStatus,
    ProvisioningState,
    PublicNetworkAccess,
    RegionalEndpoints,
    RegistryUsageUnit,
    ResourceIdentityType,
    RoleAssignmentMode,
    SkuName,
    SkuTier,
    StorageAccessMode,
    SyncState,
    SyncTrigger,
    TlsStatus,
    TokenCertificateName,
    TokenPasswordName,
    TokenStatus,
    TriggerStatus,
    TrustPolicyType,
    WebhookAction,
    WebhookStatus,
    ZoneRedundancy,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ActivationProperties",
    "Actor",
    "Archive",
    "ArchivePackageSourceProperties",
    "ArchiveProperties",
    "ArchiveUpdateParameters",
    "ArchiveUpdateProperties",
    "ArchiveVersion",
    "ArchiveVersionProperties",
    "AuthCredential",
    "AzureADAuthenticationAsArmPolicy",
    "CacheRule",
    "CacheRuleProperties",
    "CacheRuleUpdateParameters",
    "CacheRuleUpdateProperties",
    "CallbackConfig",
    "ConnectedRegistry",
    "ConnectedRegistryProperties",
    "ConnectedRegistryUpdateParameters",
    "ConnectedRegistryUpdateProperties",
    "CredentialHealth",
    "CredentialSet",
    "CredentialSetProperties",
    "CredentialSetUpdateParameters",
    "CredentialSetUpdateProperties",
    "EncryptionProperty",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "Event",
    "EventContent",
    "EventInfo",
    "EventRequestMessage",
    "EventResponseMessage",
    "ExportPipeline",
    "ExportPipelineProperties",
    "ExportPipelineTargetProperties",
    "ExportPolicy",
    "GarbageCollectionProperties",
    "GenerateCredentialsParameters",
    "GenerateCredentialsResult",
    "IPRule",
    "IdentityProperties",
    "ImportImageParameters",
    "ImportPipeline",
    "ImportPipelineProperties",
    "ImportPipelineSourceProperties",
    "ImportSource",
    "ImportSourceCredentials",
    "KeyVaultProperties",
    "LoggingProperties",
    "LoginServerProperties",
    "NetworkRuleSet",
    "OperationDefinition",
    "OperationDisplayDefinition",
    "OperationLogSpecificationDefinition",
    "OperationMetricSpecificationDefinition",
    "OperationPropertiesDefinition",
    "OperationServiceSpecificationDefinition",
    "ParentProperties",
    "PipelineRun",
    "PipelineRunProperties",
    "PipelineRunRequest",
    "PipelineRunResponse",
    "PipelineRunSourceProperties",
    "PipelineRunTargetProperties",
    "PipelineSourceTriggerDescriptor",
    "PipelineSourceTriggerProperties",
    "PipelineTriggerDescriptor",
    "PipelineTriggerProperties",
    "Policies",
    "PrivateEndpoint",
    "PrivateEndpointConnection",
    "PrivateEndpointConnectionProperties",
    "PrivateLinkResource",
    "PrivateLinkResourceProperties",
    "PrivateLinkServiceConnectionState",
    "ProgressProperties",
    "ProxyResource",
    "QuarantinePolicy",
    "RegenerateCredentialParameters",
    "Registry",
    "RegistryListCredentialsResult",
    "RegistryNameCheckRequest",
    "RegistryNameStatus",
    "RegistryPassword",
    "RegistryProperties",
    "RegistryPropertiesUpdateParameters",
    "RegistrySyncResult",
    "RegistryUpdateParameters",
    "RegistryUsage",
    "RegistryUsageListResult",
    "Replication",
    "ReplicationProperties",
    "ReplicationUpdateParameters",
    "ReplicationUpdateParametersProperties",
    "Request",
    "Resource",
    "RetentionPolicy",
    "ScopeMap",
    "ScopeMapProperties",
    "ScopeMapPropertiesUpdateParameters",
    "ScopeMapUpdateParameters",
    "Sku",
    "SoftDeletePolicy",
    "Source",
    "Status",
    "StatusDetailProperties",
    "SyncProperties",
    "SyncUpdateProperties",
    "SystemData",
    "Target",
    "TlsCertificateProperties",
    "TlsProperties",
    "Token",
    "TokenCertificate",
    "TokenCredentialsProperties",
    "TokenPassword",
    "TokenProperties",
    "TokenUpdateParameters",
    "TokenUpdateProperties",
    "TrackedResource",
    "TrustPolicy",
    "UserIdentityProperties",
    "Webhook",
    "WebhookCreateParameters",
    "WebhookProperties",
    "WebhookPropertiesCreateParameters",
    "WebhookPropertiesUpdateParameters",
    "WebhookUpdateParameters",
    "Action",
    "ActionsRequired",
    "ActivationStatus",
    "AuditLogStatus",
    "AutoGeneratedDomainNameLabelScope",
    "AzureADAuthenticationAsArmPolicyStatus",
    "CertificateType",
    "ConnectedRegistryMode",
    "ConnectionState",
    "ConnectionStatus",
    "ContainerRegistryResourceType",
    "CreatedByType",
    "CredentialHealthStatus",
    "CredentialName",
    "DefaultAction",
    "EncryptionStatus",
    "EndpointProtocol",
    "ExportPolicyStatus",
    "ImportMode",
    "LogLevel",
    "MetadataSearch",
    "NetworkRuleBypassOptions",
    "PackageSourceType",
    "PasswordName",
    "PipelineOptions",
    "PipelineRunSourceType",
    "PipelineRunTargetType",
    "PipelineSourceType",
    "PolicyStatus",
    "ProvisioningState",
    "PublicNetworkAccess",
    "RegionalEndpoints",
    "RegistryUsageUnit",
    "ResourceIdentityType",
    "RoleAssignmentMode",
    "SkuName",
    "SkuTier",
    "StorageAccessMode",
    "SyncState",
    "SyncTrigger",
    "TlsStatus",
    "TokenCertificateName",
    "TokenPasswordName",
    "TokenStatus",
    "TriggerStatus",
    "TrustPolicyType",
    "WebhookAction",
    "WebhookStatus",
    "ZoneRedundancy",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
