# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models_py3 import (  # type: ignore
    ArmIdentity,
    ArmUserIdentity,
    CertificateBodyDescription,
    CertificateDescription,
    CertificateListDescription,
    CertificateProperties,
    CertificatePropertiesWithNonce,
    CertificateVerificationDescription,
    CertificateWithNonceDescription,
    CloudToDeviceProperties,
    EncryptionPropertiesDescription,
    EndpointHealthData,
    EndpointHealthDataListResult,
    EnrichmentProperties,
    ErrorDetails,
    EventHubConsumerGroupBodyDescription,
    EventHubConsumerGroupInfo,
    EventHubConsumerGroupName,
    EventHubConsumerGroupsListResult,
    EventHubProperties,
    ExportDevicesRequest,
    FailoverInput,
    FallbackRouteProperties,
    FeedbackProperties,
    GroupIdInformation,
    GroupIdInformationProperties,
    ImportDevicesRequest,
    IotHubCapacity,
    IotHubDescription,
    IotHubDescriptionListResult,
    IotHubLocationDescription,
    IotHubNameAvailabilityInfo,
    IotHubProperties,
    IotHubPropertiesDeviceStreams,
    IotHubQuotaMetricInfo,
    IotHubQuotaMetricInfoListResult,
    IotHubSkuDescription,
    IotHubSkuDescriptionListResult,
    IotHubSkuInfo,
    IpFilterRule,
    JobResponse,
    JobResponseListResult,
    KeyVaultKeyProperties,
    ManagedIdentity,
    MatchedRoute,
    MessagingEndpointProperties,
    Name,
    NetworkRuleSetIpRule,
    NetworkRuleSetProperties,
    Operation,
    OperationDisplay,
    OperationInputs,
    OperationListResult,
    PrivateEndpoint,
    PrivateEndpointConnection,
    PrivateEndpointConnectionProperties,
    PrivateLinkResources,
    PrivateLinkServiceConnectionState,
    RegistryStatistics,
    Resource,
    RootCertificateProperties,
    RouteCompilationError,
    RouteErrorPosition,
    RouteErrorRange,
    RouteProperties,
    RoutingCosmosDBSqlApiProperties,
    RoutingEndpoints,
    RoutingEventHubProperties,
    RoutingMessage,
    RoutingProperties,
    RoutingServiceBusQueueEndpointProperties,
    RoutingServiceBusTopicEndpointProperties,
    RoutingStorageContainerProperties,
    RoutingTwin,
    RoutingTwinProperties,
    SharedAccessSignatureAuthorizationRule,
    SharedAccessSignatureAuthorizationRuleListResult,
    StorageEndpointProperties,
    SystemData,
    TagsResource,
    TestAllRoutesInput,
    TestAllRoutesResult,
    TestRouteInput,
    TestRouteResult,
    TestRouteResultDetails,
    UserSubscriptionQuota,
    UserSubscriptionQuotaListResult,
)

from ._iot_hub_client_enums import (  # type: ignore
    AccessRights,
    AuthenticationType,
    Capabilities,
    CreatedByType,
    DefaultAction,
    EndpointHealthStatus,
    IotHubNameUnavailabilityReason,
    IotHubReplicaRoleType,
    IotHubScaleType,
    IotHubSku,
    IotHubSkuTier,
    IpFilterActionType,
    IpVersion,
    JobStatus,
    JobType,
    NetworkRuleIPAction,
    PrivateLinkServiceConnectionStatus,
    PublicNetworkAccess,
    ResourceIdentityType,
    RouteErrorSeverity,
    RoutingSource,
    RoutingStorageContainerPropertiesEncoding,
    TestResultStatus,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ArmIdentity",
    "ArmUserIdentity",
    "CertificateBodyDescription",
    "CertificateDescription",
    "CertificateListDescription",
    "CertificateProperties",
    "CertificatePropertiesWithNonce",
    "CertificateVerificationDescription",
    "CertificateWithNonceDescription",
    "CloudToDeviceProperties",
    "EncryptionPropertiesDescription",
    "EndpointHealthData",
    "EndpointHealthDataListResult",
    "EnrichmentProperties",
    "ErrorDetails",
    "EventHubConsumerGroupBodyDescription",
    "EventHubConsumerGroupInfo",
    "EventHubConsumerGroupName",
    "EventHubConsumerGroupsListResult",
    "EventHubProperties",
    "ExportDevicesRequest",
    "FailoverInput",
    "FallbackRouteProperties",
    "FeedbackProperties",
    "GroupIdInformation",
    "GroupIdInformationProperties",
    "ImportDevicesRequest",
    "IotHubCapacity",
    "IotHubDescription",
    "IotHubDescriptionListResult",
    "IotHubLocationDescription",
    "IotHubNameAvailabilityInfo",
    "IotHubProperties",
    "IotHubPropertiesDeviceStreams",
    "IotHubQuotaMetricInfo",
    "IotHubQuotaMetricInfoListResult",
    "IotHubSkuDescription",
    "IotHubSkuDescriptionListResult",
    "IotHubSkuInfo",
    "IpFilterRule",
    "JobResponse",
    "JobResponseListResult",
    "KeyVaultKeyProperties",
    "ManagedIdentity",
    "MatchedRoute",
    "MessagingEndpointProperties",
    "Name",
    "NetworkRuleSetIpRule",
    "NetworkRuleSetProperties",
    "Operation",
    "OperationDisplay",
    "OperationInputs",
    "OperationListResult",
    "PrivateEndpoint",
    "PrivateEndpointConnection",
    "PrivateEndpointConnectionProperties",
    "PrivateLinkResources",
    "PrivateLinkServiceConnectionState",
    "RegistryStatistics",
    "Resource",
    "RootCertificateProperties",
    "RouteCompilationError",
    "RouteErrorPosition",
    "RouteErrorRange",
    "RouteProperties",
    "RoutingCosmosDBSqlApiProperties",
    "RoutingEndpoints",
    "RoutingEventHubProperties",
    "RoutingMessage",
    "RoutingProperties",
    "RoutingServiceBusQueueEndpointProperties",
    "RoutingServiceBusTopicEndpointProperties",
    "RoutingStorageContainerProperties",
    "RoutingTwin",
    "RoutingTwinProperties",
    "SharedAccessSignatureAuthorizationRule",
    "SharedAccessSignatureAuthorizationRuleListResult",
    "StorageEndpointProperties",
    "SystemData",
    "TagsResource",
    "TestAllRoutesInput",
    "TestAllRoutesResult",
    "TestRouteInput",
    "TestRouteResult",
    "TestRouteResultDetails",
    "UserSubscriptionQuota",
    "UserSubscriptionQuotaListResult",
    "AccessRights",
    "AuthenticationType",
    "Capabilities",
    "CreatedByType",
    "DefaultAction",
    "EndpointHealthStatus",
    "IotHubNameUnavailabilityReason",
    "IotHubReplicaRoleType",
    "IotHubScaleType",
    "IotHubSku",
    "IotHubSkuTier",
    "IpFilterActionType",
    "IpVersion",
    "JobStatus",
    "JobType",
    "NetworkRuleIPAction",
    "PrivateLinkServiceConnectionStatus",
    "PublicNetworkAccess",
    "ResourceIdentityType",
    "RouteErrorSeverity",
    "RoutingSource",
    "RoutingStorageContainerPropertiesEncoding",
    "TestResultStatus",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
